# -*- coding: utf-8 -*-
# vim:ts=4:sw=4:expandtab 2
# Copyright 2016, 2017 juga (juga at riseup dot net), MIT license.
"""Constants for the DHCP client implementation of the Anonymity Profile
([:rfc:`7844`])."""
import logging

logger = logging.getLogger(__name__)


DT_PRINT_FORMAT = '%y-%m-%d %H:%M:%S'

# client
##########
BROADCAST_MAC = 'ff:ff:ff:ff:ff:ff'
META_MAC = '00:00:00:00:00:00'
BROADCAST_ADDR = '255.255.255.255'
META_ADDR = '0.0.0.0'
CLIENT_PORT = 68
SERVER_PORT = 67

# DHCP timers
##########
LEASE_TIME = 1209600  # 14 DAYS
RENEWING_TIME = 604800  # 7 DAYS
REBINDING_TIME = 1058400  # 12 DAYS
DELAY_SELECTING = 10
TIMEOUT_SELECTING = 60
TIMEOUT_REQUESTING = 60
TIMEOUT_REQUEST_RENEWING = 226800
TIMEOUT_REQUEST_REBINDING = 75600

MAX_DELAY_SELECTING = 10
RENEW_PERC = 0.5
REBIND_PERC = 0.875

# DHCP number packet retransmissions
MAX_ATTEMPTS_DISCOVER = 5
MAX_OFFERS_COLLECTED = 1
MAX_ATTEMPTS_REQUEST = 5

# DHCP packet
##############
DHCP_OFFER_OPTIONS = [
    'server_id', 'subnet_mask', 'broadcast_address',
    'router', 'domain', 'name_server', 'lease_time', 'renewal_time',
    'rebinding_time']

# DHCP FSM
#############
STATE_ERROR = -1
STATE_PREINIT = 0
STATE_INIT = 1
STATE_SELECTING = 2
STATE_REQUESTING = 3
STATE_BOUND = 4
STATE_RENEWING = 5
STATE_REBINDING = 6
STATE_END = 7

STATES2NAMES = {
    STATE_ERROR: 'ERROR',
    STATE_PREINIT: 'PREINIT',
    STATE_INIT: 'INIT',
    STATE_SELECTING: 'SELECTING',
    STATE_REQUESTING: 'REQUESTING',
    STATE_BOUND: 'BOUND',
    STATE_RENEWING: 'RENEWING',
    STATE_REBINDING: 'REBINDING',
    STATE_END: 'END',
}

# NM integration
#####################
REASONS_NM = ['bound', 'renew', 'rebind', 'expiry', 'fail', 'timeout',
              'nak', 'end', 'abend']

REASONS_CL = ['MEDIUM', 'PREINIT', 'BOUND', 'RENEW', 'REBIND', 'REBOOT',
              'EXPIRE', 'FAIL', 'STOP', 'RELEASE', 'NBI', 'TIMEOUT']

STATES2REASONS = {
    STATE_PREINIT: 'PREINIT',
    STATE_INIT: 'INIT',
    STATE_SELECTING: 'SELECTING',
    STATE_BOUND: 'BOUND',
    STATE_END: 'END',
    STATE_REBINDING: 'REBIND',
    STATE_RENEWING: 'RENEW',
    STATE_ERROR: "FAIL",
    # NOTE: there could be implemented a way toknow the reason for failure so
    # that it can be passed to NetworkManager, as dhclient does, ie:
    # "STOP", "EXPIRE"
}

# systemd events
DHCP_EVENTS = {
    'STOP': 0,
    'IP_ACQUIRE': 1,
    'IP_CHANGE': 2,
    'EXPIRED': 3,
    'RENEW': 4,
}

SCRIPT_ENV_KEYS = ['reason', 'medium', 'interface',
                   # 'client', 'pid',
                   'new_ip_address', 'new_subnet_mask', 'new_network_number',
                   'new_domain_name_servers', 'new_domain_name', 'new_routers',
                   'new_broadcast_address', 'new_next_server',
                   'new_dhcp_server_id']

LEASEATTRS_SAMEAS_ENVKEYS = ['interface']  # , 'reason']
# 'client', 'pid',
# these are not set as environment but put in lease file
# , 'rebind', 'renew', 'expiry'

LEASEATTRS2ENVKEYS = {
    'address': 'new_ip_address',
    'subnet_mask': 'new_subnet_mask',
    'broadcast_address': 'new_broadcast_address',
    'next_server': 'new_next_server',
    'server_id': 'new_server_id',

    'network': 'new_network_number',
    'domain': 'new_domain_name',
    'name_server': 'new_domain_name_servers',
    'router': 'new_routers',
}

LEASE_ATTRS2LEASE_FILE = {
    'interface': 'interface',

    'address': 'fixed-address',

    'subnet_mask': 'option subnet-mask',
    'broadcast_address': 'option broadcast-address',

    'domain': 'option domain-name',
    'name_server': 'option domain-name-servers',
    'router': 'option routers',

    'lease_time': 'option dhcp-lease-time',
    'rebinding_time': 'option dhcp-rebinding-time',
    'renewal_time': 'option dhcp-renewal-time',
    'server_id': 'option dhcp-server-identifier',
    'renew': 'renew',
    'rebind': 'rebind',
    'expiry': 'expiry',
}

LEASE_ATTRS2LEASE_LOG = {
    'interface': 'interface',

    'subnet_mask': 'option subnet_mask',
    'broadcast_address': 'option broadcast_address',

    'address': 'ip_address',

    'router': 'option routers',
    'domain': 'option domain_name',
    'name_server': 'option domain_name_servers',

    'lease_time': 'option dhcp_lease_time',
    'renewal_time': 'option dhcp_renewal_time',
    'rebinding_time': 'option dhcp_rebinding_time',
    'server_id': 'option dhcp_server_identifier',
    'expiry': 'expiry',
}

ENV_OPTIONS_REQ = {
    'requested_subnet_mask': '1',
    'requested_router': '1',
    'requested_domain_name_server': '1',
    'requested_domain_name': '1',
    'requested_router_discovery': '1',
    'requested_static_route': '1',
    'requested_vendor_specific': '1',
    'requested_netbios_nameserver': '1',
    'requested_netbios_node_type': '1',
    'requested_netbios_scope': '1',
    'requested_classless_static_route_option': '1',
    'requested_private_classless_static_route': '1',
    'requested_private_proxy_autodiscovery': '1',
}

PRL = b"\x01\x03\x06\x0f\x1f\x21\x2b\x2c\x2e\x2f\x79\xf9\xfc"
"""
SD_DHCP_OPTION_SUBNET_MASK                     = 1
SD_DHCP_OPTION_ROUTER                          = 3
SD_DHCP_OPTION_DOMAIN_NAME_SERVER              = 6
SD_DHCP_OPTION_DOMAIN_NAME                     = 15
SD_DHCP_OPTION_ROUTER_DISCOVER                 = 31
SD_DHCP_OPTION_STATIC_ROUTE                    = 33
SD_DHCP_OPTION_VENDOR_SPECIFIC                 = 43
SD_DHCP_OPTION_NETBIOS_NAMESERVER              = 44
SD_DHCP_OPTION_NETBIOS_NODETYPE                = 46
SD_DHCP_OPTION_NETBIOS_SCOPE                   = 47
SD_DHCP_OPTION_CLASSLESS_STATIC_ROUTE          = 121
SD_DHCP_OPTION_PRIVATE_CLASSLESS_STATIC_ROUTE  = 249
SD_DHCP_OPTION_PRIVATE_PROXY_AUTODISCOVERY     = 252
"""

FSM_ATTRS = ['request_attempts', 'discover_attempts', 'script',
             'time_sent_request', 'current_state', 'client']

XID_MIN = 1
XID_MAX = 900000000

SCRIPT_PATH = '/sbin/dhcpcanon-script'
PID_PATH = '/var/run/dhcpcanon.pid'
LEASE_PATH = '/var/lib/dhcp/dhcpcanon.leases'
CONF_PATH = '/etc/dhcp/dhcpcanon.conf'
RESOLVCONF = '/sbin/resolvconf'
RESOLVCONF_ADMIN = '/usr/bin/resolvconf-admin'



# -*- coding: utf-8 -*-
# vim:ts=4:sw=4:expandtab 2
# Copyright 2016, 2017 juga (juga at riseup dot net), MIT license.
"""Timers for the DHCP client implementation of the Anonymity Profile
([:rfc:`7844`])."""
from __future__ import absolute_import

import logging
import random
from datetime import datetime, timedelta

from .constants import (DT_PRINT_FORMAT, MAX_DELAY_SELECTING, REBIND_PERC,
                        RENEW_PERC)

logger = logging.getLogger(__name__)


def future_dt_str(dt, td):
    """."""
    if isinstance(td, str):
        td = float(td)
    td = timedelta(seconds=td)
    future_dt = dt + td
    return future_dt.strftime(DT_PRINT_FORMAT)


def nowutc():
    """."""
    # NOTE: Not using UTC, as all the timers are set in reference to local time
    # now = datetime.utcnow().replace(tzinfo=utc)
    now = datetime.now()
    return now


def gen_delay_selecting():
    """Generate the delay in seconds in which the DISCOVER will be sent.

    [:rfc:`2131#section-4.4.1`]::

        The client SHOULD wait a random time between one and ten seconds to
        desynchronize the use of DHCP at startup.

    """
    delay = float(random.randint(0, MAX_DELAY_SELECTING))
    logger.debug('Delay to enter in SELECTING %s.', delay)
    logger.debug('SELECTING will happen on %s',
                 future_dt_str(nowutc(), delay))
    return delay


def gen_timeout_resend(attempts):
    """Generate the time in seconds in which DHCPDISCOVER wil be retransmited.

    [:rfc:`2131#section-3.1`]::

        might retransmit the
        DHCPREQUEST message four times, for a total delay of 60 seconds

    [:rfc:`2131#section-4.1`]::

        For example, in a 10Mb/sec Ethernet
        internetwork, the delay before the first retransmission SHOULD be 4
        seconds randomized by the value of a uniform random number chosen
        from the range -1 to +1.  Clients with clocks that provide resolution
        granularity of less than one second may choose a non-integer
        randomization value.  The delay before the next retransmission SHOULD
        be 8 seconds randomized by the value of a uniform number chosen from
        the range -1 to +1.  The retransmission delay SHOULD be doubled with
        subsequent retransmissions up to a maximum of 64 seconds.

    """
    timeout = 2 ** (attempts + 1) + random.uniform(-1, +1)
    logger.debug('next timeout resending will happen on %s',
                 future_dt_str(nowutc(), timeout))
    return timeout


def gen_timeout_request_renew(lease):
    """Generate time in seconds to retransmit DHCPREQUEST.

    [:rfc:`2131#section-4..4.5`]::

        In both RENEWING and REBINDING states,
        if the client receives no response to its DHCPREQUEST
        message, the client SHOULD wait one-half of the remaining
        time until T2 (in RENEWING state) and one-half of the
        remaining lease time (in REBINDING state), down to a
        minimum of 60 seconds, before retransmitting the
        DHCPREQUEST message.

    """
    time_left = (lease.rebinding_time - lease.renewing_time) * RENEW_PERC
    if time_left < 60:
        time_left = 60
    logger.debug('Next request in renew will happen on %s',
                 future_dt_str(nowutc(), time_left))
    return time_left


def gen_timeout_request_rebind(lease):
    """."""
    time_left = (lease.lease_time - lease.rebinding_time) * RENEW_PERC
    if time_left < 60:
        time_left = 60
    logger.debug('Next request on rebinding will happen on %s',
                 future_dt_str(nowutc(), time_left))
    return time_left


def gen_renewing_time(lease_time, elapsed=0):
    """Generate RENEWING time.

    [:rfc:`2131#section-4.4.5`]::

        T1
        defaults to (0.5 * duration_of_lease).  T2 defaults to (0.875 *
        duration_of_lease).  Times T1 and T2 SHOULD be chosen with some
        random "fuzz" around a fixed value, to avoid synchronization of
        client reacquisition.

    """
    renewing_time = int(lease_time) * RENEW_PERC - elapsed
    # FIXME:80 [:rfc:`2131#section-4.4.5`]: the chosen "fuzz" could fingerprint
    # the implementation
    # NOTE: here using same "fuzz" as systemd?
    range_fuzz = int(lease_time) * REBIND_PERC - renewing_time
    logger.debug('rebinding fuzz range %s', range_fuzz)
    fuzz = random.uniform(-(range_fuzz),
                          +(range_fuzz))
    renewing_time += fuzz
    logger.debug('Renewing time %s.', renewing_time)
    return renewing_time


def gen_rebinding_time(lease_time, elapsed=0):
    """."""
    rebinding_time = int(lease_time) * REBIND_PERC - elapsed
    # FIXME:90 [:rfc:`2131#section-4.4.5`]: the chosen "fuzz" could fingerprint
    # the implementation
    # NOTE: here using same "fuzz" as systemd?
    range_fuzz = int(lease_time) - rebinding_time
    logger.debug('rebinding fuzz range %s', range_fuzz)
    fuzz = random.uniform(-(range_fuzz),
                          +(range_fuzz))
    rebinding_time += fuzz
    logger.debug('Rebinding time %s.', rebinding_time)
    return rebinding_time



# -*- coding: utf-8 -*-
# vim:ts=4:sw=4:expandtab
# Copyright 2016, 2017 juga (juga at riseup dot net), MIT license.
"""DCHP client implementation of the Anonymity Profiles [:rfc:`7844`]."""
from __future__ import absolute_import, unicode_literals

import logging

from netaddr import AddrFormatError
from scapy.arch import get_if_raw_hwaddr
from scapy.automaton import ATMT, Automaton
from scapy.config import conf
from scapy.sendrecv import sendp
from scapy.utils import str2mac

from .clientscript import ClientScript
from .constants import (CLIENT_PORT, DELAY_SELECTING, FSM_ATTRS, LEASE_TIME,
                        MAX_ATTEMPTS_DISCOVER, MAX_ATTEMPTS_REQUEST,
                        MAX_OFFERS_COLLECTED, REBINDING_TIME, RENEWING_TIME,
                        SERVER_PORT, STATE_BOUND, STATE_END, STATE_ERROR,
                        STATE_INIT, STATE_PREINIT, STATE_REBINDING,
                        STATE_RENEWING, STATE_REQUESTING, STATE_SELECTING,
                        STATES2NAMES, TIMEOUT_REQUEST_REBINDING,
                        TIMEOUT_REQUEST_RENEWING, TIMEOUT_REQUESTING,
                        TIMEOUT_SELECTING)
from .dhcpcap import DHCPCAP
from .dhcpcaputils import isack, isnak, isoffer
from .timers import (gen_delay_selecting, gen_timeout_request_rebind,
                     gen_timeout_request_renew, gen_timeout_resend, nowutc)
from .netutils import set_net

logger = logging.getLogger(__name__)


class DHCPCAPFSM(Automaton):
    """DHCP client Finite State Machine (FSM)."""

    def dict_self(self):
        """Return the self object attributes not inherited as dict."""
        return {k: v for k, v in self.__dict__.items() if k in FSM_ATTRS}

    def __str__(self):
        return str(self.dict_self())

    def __eq__(self, other):
        return self.__dict__ == other.__dict__

    def reset(self, iface=None, client_mac=None, xid=None, scriptfile=None):
        """Reset object attributes when state is INIT."""
        logger.debug('Reseting attributes.')
        if iface is None:
            iface = conf.iface
        if client_mac is None:
            # scapy for python 3 returns byte, not tuple
            tempmac = get_if_raw_hwaddr(iface)
            if isinstance(tempmac, tuple) and len(tempmac) == 2:
                mac = tempmac[1]
            else:
                mac = tempmac
            client_mac = str2mac(mac)
        self.client = DHCPCAP(iface=iface, client_mac=client_mac, xid=xid)
        if scriptfile is not None:
            self.script = ClientScript(scriptfile)
        else:
            self.script = None
        self.time_sent_request = None
        self.discover_attempts = 0
        self.request_attempts = 0
        self.current_state = STATE_PREINIT
        self.offers = list()

    def __init__(self, iface=None, server_port=None,
                 client_port=None, client_mac=None, xid=None,
                 scriptfile=None, delay_selecting=False,
                 delay_before_selecting=None,
                 timeout_select=None, debug_level=5, *args, **kargs):
        """Overwrites Automaton __init__ method.

        [ :rfc:`7844#section-3.4` ] ::
            If the hardware address is reset to a new
            randomized value, the DHCP client SHOULD use the new randomized
            value in the DHCP messages

        """
        logger.debug('Inizializating FSM.')
        super(DHCPCAPFSM, self).__init__(*args, **kargs)
        self.debug_level = debug_level
        self.delay_selecting = delay_selecting
        self.delay_before_selecting = delay_before_selecting
        self.timeout_select = timeout_select
        self.reset(iface, client_mac, xid, scriptfile)
        self.client.server_port = server_port or SERVER_PORT
        self.client.client_port = client_port or CLIENT_PORT
        self.socket_kargs = {
            'filter': 'udp and src port {0} and dst port {1}'
                      ' and ether dst {2}'.
                      format(self.client.server_port,
                             self.client.client_port,
                             self.client.client_mac)
        }
        if self.script is not None:
            self.script.script_init(self.client.lease, self.current_state)
            self.script.script_go()
        logger.debug('FSM thread id: %s.', self.threadid)

    def get_timeout(self, state, function):
        """Workaround to get timeout in the ATMT.timeout class method."""
        state = STATES2NAMES[state]
        for timeout_fn_t in self.timeout[state]:
            # access the function name
            if timeout_fn_t[1] is not None and \
               timeout_fn_t[1].atmt_condname == function.atmt_condname:
                logger.debug('Timeout for state %s, function %s, is %s',
                             state, function.atmt_condname, timeout_fn_t[0])
                return timeout_fn_t[0]
        return None

    def set_timeout(self, state, function, newtimeout):
        """
        Workaround to change timeout values in the ATMT.timeout class method.

        self.timeout format is::

            {'STATE': [
                (TIMEOUT0, <function foo>),
                (TIMEOUT1, <function bar>)),
                (None, None)
                ],
            }


        """
        state = STATES2NAMES[state]
        for timeout_fn_t in self.timeout[state]:
            # access the function name
            if timeout_fn_t[1] is not None and \
               timeout_fn_t[1].atmt_condname == function.atmt_condname:
                # convert list to tuple to make it mutable
                timeout_l = list(timeout_fn_t)
                # modify the timeout
                timeout_l[0] = newtimeout
                # set the new timeoute to self.timeout
                i = self.timeout[state].index(timeout_fn_t)
                self.timeout[state][i] = tuple(timeout_l)
                logger.debug('Set state %s, function %s, to timeout %s',
                             state, function.atmt_condname, newtimeout)

    def send_discover(self):
        """Send discover."""
        assert self.client
        assert self.current_state == STATE_INIT or \
            self.current_state == STATE_SELECTING
        pkt = self.client.gen_discover()
        sendp(pkt)
        # FIXME:20 check that this is correct,: all or only discover?
        if self.discover_attempts < MAX_ATTEMPTS_DISCOVER:
            self.discover_attempts += 1
        timeout = gen_timeout_resend(self.discover_attempts)
        self.set_timeout(self.current_state,
                         self.timeout_selecting,
                         timeout)
        # logger.info('DHCPDISCOVER on %s to %s port %s' %
        #             (self.client.iface, self.client.server_mac,
        #              self.client.server_port)))

    def select_offer(self):
        """Select an offer from the offers received.

        [:rfc:`2131#section-4.2`]::

            DHCP clients are free to use any strategy in selecting a DHCP
            server among those from which the client receives a DHCPOFFER.

        [:rfc:`2131#section-4.4.1`]::

            The time
            over which the client collects messages and the mechanism used to
            select one DHCPOFFER are implementation dependent.

        Nor [:rfc:`7844`] nor [:rfc:`2131`] specify the algorithm.
        Here, currently the first offer is selected.

        .. todo::

           - Check other implementations algorithm to select offer.

        """
        logger.debug('Selecting offer.')
        pkt = self.offers[0]
        self.client.handle_offer(pkt)

    def send_request(self):
        """Send request.

        [:rfc:`2131#section-3.1`]::

        a client retransmitting as described in section 4.1 might retransmit
        the DHCPREQUEST message four times, for a total delay of 60 seconds

        .. todo::

           - The maximum number of retransmitted REQUESTs is per state or in
             total?
           - Are the retransmitted REQUESTs independent to the retransmitted
             DISCOVERs?

        """
        assert self.client
        if self.current_state == STATE_BOUND:
            pkt = self.client.gen_request_unicast()
        else:
            pkt = self.client.gen_request()
        sendp(pkt)
        logger.debug('Modifying FSM obj, setting time_sent_request.')
        self.time_sent_request = nowutc()
        logger.info('DHCPREQUEST of %s on %s to %s port %s',
                    self.client.iface, self.client.client_ip,
                    self.client.server_ip, self.client.server_port)

        # NOTE: see previous TODO, maybe the MAX_ATTEMPTS_REQUEST needs to be
        # calculated per state.
        if self.request_attempts < MAX_ATTEMPTS_REQUEST:
            self.request_attempts *= 2
            logger.debug('Increased request attempts to %s',
                         self.request_attempts)
        if self.current_state == STATE_RENEWING:
            timeout_renewing = gen_timeout_request_renew(self.client.lease)
            self.set_timeout(self.current_state,
                             self.timeout_request_renewing,
                             timeout_renewing)
        elif self.current_state == STATE_REBINDING:
            timeout_rebinding = gen_timeout_request_rebind(self.client.lease)
            self.set_timeout(self.current_state,
                             self.timeout_request_rebinding,
                             timeout_rebinding)
        else:
            timeout_requesting = \
                gen_timeout_resend(self.request_attempts)
            self.set_timeout(self.current_state,
                             self.timeout_requesting,
                             timeout_requesting)

    def set_timers(self):
        """Set renewal, rebinding times."""
        logger.debug('setting timeouts')
        self.set_timeout(self.current_state,
                         self.renewing_time_expires,
                         self.client.lease.renewal_time)
        self.set_timeout(self.current_state,
                         self.rebinding_time_expires,
                         self.client.lease.rebinding_time)

    def process_received_ack(self, pkt):
        """Process a received ACK packet.

        Not specifiyed in [:rfc:`7844`].
        Probe the offered IP in [:rfc:`2131#section-2.2.`]::

            the allocating
            server SHOULD probe the reused address before allocating the
            address, e.g., with an ICMP echo request, and the client SHOULD
            probe the newly received address, e.g., with ARP.

            The client SHOULD broadcast an ARP
            reply to announce the client's new IP address and clear any
            outdated ARP cache entries in hosts on the client's subnet.

        It is also not specifiyed in [:rfc:`7844`] nor [:rfc:`2131`] how to
        check that the offered IP is valid.

        .. todo::
           - Check that nor ``dhclient`` nor ``systemd-networkd`` send an ARP.
           - Check how other implementations check that the ACK paremeters
             are valid, ie, if the ACK fields match the fields in the OFFER.
           - Check to which state the client should go back to when the
             offered parameters are not valid.

        """
        if isack(pkt):
            try:
                self.event = self.client.handle_ack(pkt,
                                                    self.time_sent_request)
            except AddrFormatError as err:
                logger.error(err)
                # NOTE: see previous TODO, maybe should go back to other state.
                raise self.SELECTING()
            # NOTE: see previous TODO, not checking address with ARP.
            logger.info('DHCPACK of %s from %s' %
                        (self.client.client_ip, self.client.server_ip))
            return True
        return False

    def process_received_nak(self, pkt):
        """Process a received NAK packet."""
        if isnak(pkt):
            logger.info('DHCPNAK of %s from %s',
                        self.client.client_ip, self.client.server_ip)
            return True
        return False

    #################################################################
    # State machine
    #################################################################

    # STATES
    #########

    @ATMT.state(initial=1)
    def INIT(self):
        """INIT state.

        [:rfc:`2131#section-4.4.1`]::

            The client SHOULD wait a random time between one and ten
            seconds to desynchronize the use of DHCP at startup

        .. todo::
           - The initial delay is implemented, but probably is not in other
             implementations. Check what other implementations do.
        """
        # NOTE: in case INIT is reached from other state, initialize attributes
        # reset all variables.
        logger.debug('In state: INIT')
        if self.current_state is not STATE_PREINIT:
            self.reset()
        self.current_state = STATE_INIT
        # NOTE: see previous TODO, maybe this is not needed.
        if self.delay_selecting:
            if self.delay_before_selecting is None:
                delay_before_selecting = gen_delay_selecting()
            else:
                delay_before_selecting = self.delay_before_selecting
        else:
            delay_before_selecting = 0
        self.set_timeout(self.current_state,
                         self.timeout_delay_before_selecting,
                         delay_before_selecting)
        if self.timeout_select is not None:
            self.set_timeout(STATE_SELECTING,
                             self.timeout_selecting,
                             self.timeout_select)

    @ATMT.state()
    def SELECTING(self):
        """SELECTING state."""
        logger.debug('In state: SELECTING')
        self.current_state = STATE_SELECTING

    @ATMT.state()
    def REQUESTING(self):
        """REQUESTING state."""
        logger.debug('In state: REQUESTING')
        self.current_state = STATE_REQUESTING

    @ATMT.state()
    def BOUND(self):
        """BOUND state."""
        logger.debug('In state: BOUND')
        logger.info('(%s) state changed %s -> bound', self.client.iface,
                    STATES2NAMES[self.current_state])
        self.current_state = STATE_BOUND
        self.client.lease.info_lease()
        if self.script is not None:
            self.script.script_init(self.client.lease, self.current_state)
            self.script.script_go()
        else:
            try:
                set_net(self.client.lease)
            except Exception as e:
                logger.error('Can not set IP', exc_info=True)
                # raise self.END()
        # TODO: go daemon?

    @ATMT.state()
    def RENEWING(self):
        """RENEWING state."""
        logger.debug('In state: RENEWING')
        self.current_state = STATE_RENEWING
        if self.script is not None:
            self.script.script_init(self.client.lease, self.current_state)
            self.script.script_go()
        else:
            set_net(self.client.lease)

    @ATMT.state()
    def REBINDING(self):
        """REBINDING state."""
        logger.debug('In state: REBINDING')
        self.current_state = STATE_REBINDING
        if self.script is not None:
            self.script.script_init(self.client.lease, self.current_state)
            self.script.script_go()
        else:
            set_net(self.client.lease)

    @ATMT.state(final=1)
    def END(self):
        """END state."""
        logger.debug('In state: END')
        self.current_state = STATE_END
        if self.script is not None:
            self.script.script_init(self.client.lease, self.current_state)
            self.script.script_go()
        else:
            set_net(self.client.lease)
        return

    @ATMT.state(error=1)
    def ERROR(self):
        """ERROR state."""
        logger.debug('In state: ERROR')
        self.current_state = STATE_ERROR
        if self.script is not None:
            self.script.script_init(self.client.lease, self.current_state)
            self.script.script_go()
        set_net(self.client.lease)
        raise self.INIT()

    # TIMEOUTS
    ###########

    # TIMEOUTS: retransmissions
    # ----------------------------
    @ATMT.timeout(INIT, DELAY_SELECTING)
    def timeout_delay_before_selecting(self):
        """Timeout delay selecting in INIT state."""
        logger.debug('C1:T. In %s, timeout delay selecting, raise SELECTING',
                     self.current_state)
        raise self.SELECTING()

    @ATMT.timeout(SELECTING, TIMEOUT_SELECTING)
    def timeout_selecting(self):
        """Timeout of selecting on SELECTING state.

        Not specifiyed in [:rfc:`7844`].
        See comments in :func:`dhcpcapfsm.DHCPCAPFSM.timeout_request`.

        """
        logger.debug('C2.1: T In %s, timeout receiving response to select.',
                     self.current_state)

        if len(self.offers) >= MAX_OFFERS_COLLECTED:
            logger.debug('C2.2: T Maximum number of offers reached, '
                         'raise REQUESTING.')
            raise self.REQUESTING()

        if self.discover_attempts >= MAX_ATTEMPTS_DISCOVER:
            logger.debug('C2.3: T Maximum number of discover retries is %s'
                         ' and already sent %s.',
                         MAX_ATTEMPTS_DISCOVER, self.discover_attempts)
            if len(self.offers) <= 0:
                logger.debug('C2.4: T. But no OFFERS where received, '
                             'raise ERROR.')
                raise self.ERROR()
            logger.debug('C2.4: F. But there is some OFFERS, '
                         'raise REQUESTING.')
            raise self.REQUESTING()

        logger.debug('C2.2: F. Still not received all OFFERS, but not '
                     'max # attemps reached, raise SELECTING.')
        raise self.SELECTING()

    @ATMT.timeout(REQUESTING, TIMEOUT_REQUESTING)
    def timeout_requesting(self):
        """Timeout requesting in REQUESTING state.

        Not specifiyed in [:rfc:`7844`]

        [:rfc:`2131#section-3.1`]::

            might retransmit the
            DHCPREQUEST message four times, for a total delay of 60 seconds

        """
        logger.debug("C3.2: T. In %s, timeout receiving response to request, ",
                     self.current_state)
        if self.discover_requests >= MAX_ATTEMPTS_REQUEST:
            logger.debug('C2.3: T. Maximum number %s of REQUESTs '
                         'reached, already sent %s, raise ERROR.',
                         MAX_ATTEMPTS_REQUEST, self.disover_requests)
            raise self.ERROR()
        logger.debug("C2.3: F. Maximum number of REQUESTs retries not reached,"
                     "raise REQUESTING.")
        raise self.REQUESTING()

    @ATMT.timeout(RENEWING, TIMEOUT_REQUEST_RENEWING)
    def timeout_request_renewing(self):
        """Timeout of renewing on RENEWING state.

        Same comments as in
        :func:`dhcpcapfsm.DHCPCAPFSM.timeout_requesting`.

        """
        logger.debug("C5.2:T In %s, timeout receiving response to request.",
                     self.current_state)
        if self.request_attempts >= MAX_ATTEMPTS_REQUEST:
            logger.debug('C2.3: T Maximum number %s of REQUESTs '
                         'reached, already sent %s, wait to rebinding time.',
                         MAX_ATTEMPTS_REQUEST, self.disover_requests)
            # raise self.ERROR()
        logger.debug("C2.3: F. Maximum number of REQUESTs retries not reached,"
                     "raise RENEWING.")
        raise self.RENEWING()

    @ATMT.timeout(REBINDING, TIMEOUT_REQUEST_REBINDING)
    def timeout_request_rebinding(self):
        """Timeout of request rebinding on REBINDING state.

        Same comments as in
        :func:`dhcpcapfsm.DHCPCAPFSM.timeout_requesting`.

        """
        logger.debug("C6.2:T In %s, timeout receiving response to request.",
                     self.current_state)
        if self.request_attempts >= MAX_ATTEMPTS_REQUEST:
            logger.debug('C.2.3: T. Maximum number %s of REQUESTs '
                         'reached, already sent %s, wait lease time expires.',
                         MAX_ATTEMPTS_REQUEST, self.disover_requests)
            # raise self.ERROR()
        logger.debug("C2.3: F. Maximum number of REQUESTs retries not reached,"
                     "raise REBINDING.")
        raise self.REBINDING()

    # TIMEOUTS: timers
    # -----------------

    @ATMT.timeout(BOUND, RENEWING_TIME)
    def renewing_time_expires(self):
        """Timeout renewing time (T1), transition to RENEWING."""
        logger.debug("C4. Timeout renewing time, in BOUND state, "
                     "raise RENEWING.")
        raise self.RENEWING()

    @ATMT.timeout(RENEWING, REBINDING_TIME)
    def rebinding_time_expires(self):
        """Timeout rebinding time (T2), transition to REBINDING."""
        logger.debug("C5.3. Timeout rebinding time, in RENEWING state, "
                     "raise REBINDING.")
        raise self.REBINDING()

    @ATMT.timeout(REBINDING, LEASE_TIME)
    def lease_expires(self):
        """Timeout lease time, transition to INIT.

        Not sending DHCPRELEASE to minimize deanonymization

        [:rfc:`2131#section-4.4.6`]::

            Note that the correct operation
            of DHCP does not depend on the transmission of DHCPRELEASE.

        """
        logger.debug("C6.3. Timeout lease time, in REBINDING state, "
                     "raise INIT.")
        raise self.STATE_INIT()

    # RECEIVE CONDITIONS
    ####################

    @ATMT.receive_condition(SELECTING)
    def receive_offer(self, pkt):
        """Receive offer on SELECTING state."""
        logger.debug("C2. Received OFFER?, in SELECTING state.")
        if isoffer(pkt):
            logger.debug("C2: T, OFFER received")
            self.offers.append(pkt)
            if len(self.offers) >= MAX_OFFERS_COLLECTED:
                logger.debug("C2.5: T, raise REQUESTING.")
                self.select_offer()
                raise self.REQUESTING()
            logger.debug("C2.5: F, raise SELECTING.")
            raise self.SELECTING()

    # same as:, but would can not be overloaded
    # @ATMT.receive_condition(RENEWING)
    # @ATMT.receive_condition(REBINDING)
    @ATMT.receive_condition(REQUESTING)
    def receive_ack_requesting(self, pkt):
        """Receive ACK in REQUESTING state."""
        logger.debug("C3. Received ACK?, in REQUESTING state.")
        if self.process_received_ack(pkt):
            logger.debug("C3: T. Received ACK, in REQUESTING state, "
                         "raise BOUND.")
            raise self.BOUND()

    # same as:, but would can not be overloaded
    # @ATMT.receive_condition(RENEWING)
    # @ATMT.receive_condition(REBINDING)
    @ATMT.receive_condition(REQUESTING)
    def receive_nak_requesting(self, pkt):
        """Receive NAK in REQUESTING state."""
        logger.debug("C3.1. Received NAK?, in REQUESTING state.")
        if self.process_received_nak(pkt):
            logger.debug("C3.1: T. Received NAK, in REQUESTING state, "
                         "raise INIT.")
            raise self.INIT()

    @ATMT.receive_condition(RENEWING)
    def receive_ack_renewing(self, pkt):
        """Receive ACK in RENEWING state."""
        logger.debug("C3. Received ACK?, in RENEWING state.")
        if self.process_received_ack(pkt):
            logger.debug("C3: T. Received ACK, in RENEWING state, "
                         "raise BOUND.")
            raise self.BOUND()

    @ATMT.receive_condition(RENEWING)
    def receive_nak_renewing(self, pkt):
        """Receive NAK in RENEWING state."""
        logger.debug("C3.1. Received NAK?, in RENEWING state.")
        if self.process_received_nak(pkt):
            logger.debug("C3.1: T. Received NAK, in RENEWING state, "
                         " raise INIT.")
            raise self.INIT()

    @ATMT.receive_condition(REBINDING)
    def receive_ack_rebinding(self, pkt):
        """Receive ACK in REBINDING state."""
        logger.debug("C3. Received ACK?, in REBINDING state.")
        if self.process_received_ack(pkt):
            logger.debug("C3: T. Received ACK, in REBINDING state, "
                         "raise BOUND.")
            raise self.BOUND()

    @ATMT.receive_condition(REBINDING)
    def receive_nak_rebinding(self, pkt):
        """Receive NAK in REBINDING state."""
        logger.debug("C3.1. Received NAK?, in RENEWING state.")
        if self.process_received_nak(pkt):
            logger.debug("C3.1: T. Received NAK, in RENEWING state, "
                         "raise INIT.")
            raise self.INIT()

    # ACTIONS
    ##########

    # ACTIONS: on timeouts
    # -----------------------

    @ATMT.action(timeout_delay_before_selecting)
    @ATMT.action(timeout_selecting)
    def action_transmit_discover(self):
        """Action on timeout, send DISCOVER."""
        logger.debug('Action on timeout, in state %s: send DISCOVER.',
                     self.current_state)
        self.send_discover()

    @ATMT.action(timeout_requesting)
    @ATMT.action(timeout_request_rebinding)
    @ATMT.action(rebinding_time_expires)
    @ATMT.action(receive_offer)
    @ATMT.action(timeout_request_renewing)
    @ATMT.action(renewing_time_expires)
    def action_transmit_request(self):
        """Action on X: send REQUEST."""
        logger.debug('Action on timeout X/receive X, in state %s: '
                     'send REQUEST.', self.current_state)
        self.send_request()

    # ACTIONS: on receive conditions
    # -------------------------------

    @ATMT.action(receive_ack_rebinding)
    @ATMT.action(receive_ack_requesting)
    def on_ack_requesting(self):
        """Action on receive ACK requesting in REQUESTING state: set timers."""
        # NOTE: not recording lease
        logger.debug('Action on receive ACK in REQUESTING state: set timers.')
        self.set_timers()

    @ATMT.action(receive_ack_renewing)
    def on_renewing(self):
        """Action on renewing on RENEWING state.

        Not recording lease, but restarting timers.

        """
        self.client.lease.sanitize_net_values()
        self.client.lease.set_times(self.time_sent_request)
        self.set_timers()



# -*- coding: utf-8 -*-
# vim:ts=4:sw=4:expandtab
# Copyright 2016, 2017 juga (juga at riseup dot net), MIT license.
"""."""
from dhcpcanon.dhcpcaplease import DHCPCAPLease
from dhcpcanon.dhcpcap import DHCPCAP

# init, before send discover

# init after sent discover
client_init = DHCPCAP(iface='enp0s25', client_mac='f0:de:f1:b8:9b:db',
                      client_ip='0.0.0.0', client_port=68,
                      server_mac='ff:ff:ff:ff:ff:ff',
                      server_ip='255.255.255.255', server_port=67,
                      lease=DHCPCAPLease(address='', server_id='',
                                         next_server='', router='',
                                         subnet_mask='', broadcast_address='',
                                         domain='', name_server='', subnet='',
                                         lease_time='', renewal_time='',
                                         rebinding_time='',
                                         interface='enp0s25',
                                         subnet_mask_cidr='', network='',
                                         expiry='', renew='', rebind=''),
                      event=None)
# selecting after received offer
client_select = DHCPCAP(iface='enp0s25', client_mac='f0:de:f1:b8:9b:db',
                        client_ip='0.0.0.0', client_port=68,
                        server_mac='ff:ff:ff:ff:ff:ff',
                        server_ip='255.255.255.255', server_port=67,
                        lease=DHCPCAPLease(address='192.168.2.113',
                                           server_id='192.168.2.1',
                                           next_server='192.168.2.1',
                                           router='192.168.2.1',
                                           subnet_mask='255.255.255.0',
                                           broadcast_address='192.168.2.255',
                                           domain='localdomain',
                                           name_server='192.168.2.1 8.8.8.8',
                                           subnet='192.168.2.0',
                                           lease_time='43200',
                                           renewal_time='21600',
                                           rebinding_time='37800',
                                           interface='enp0s25',
                                           subnet_mask_cidr='24',
                                           network='',
                                           expiry='', renew='', rebind=''),
                        event=None)


# in requesting state after received ack
client_request = DHCPCAP(iface='enp0s25', client_mac='f0:de:f1:b8:9b:db',
                         client_ip='0.0.0.0', client_port=68,
                         server_mac='1c:74:0d:b2:e5:10',
                         server_ip='192.168.2.1', server_port=67,
                         lease=DHCPCAPLease(address='192.168.2.113',
                                            server_id='192.168.2.1',
                                            next_server='192.168.2.1',
                                            router='192.168.2.1',
                                            subnet_mask='255.255.255.0',
                                            broadcast_address='192.168.2.255',
                                            domain='localdomain',
                                            name_server='192.168.2.1 8.8.8.8',
                                            subnet='192.168.2.0',
                                            lease_time='43200',
                                            renewal_time='21600',
                                            rebinding_time='37800',
                                            interface='enp0s25',
                                            subnet_mask_cidr='24',
                                            network='',
                                            expiry='17-06-15 23:00:32',
                                            renew='17-06-15 17:00:32',
                                            rebind='17-06-15 21:30:32'),
                         event=4)



# -*- coding: utf-8 -*-
# vim:ts=4:sw=4:expandtab
# Copyright 2016, 2017 juga (juga at riseup dot net), MIT license.
""""""
import logging
from datetime import datetime

from dhcpcap_leases import LEASE_ACK, LEASE_REQUEST
from dhcpcap_pkts import dhcp_ack, dhcp_discover, dhcp_offer, dhcp_request

FORMAT = "%(levelname)s: %(filename)s:%(lineno)s - %(funcName)s - " + \
         "%(message)s"
logging.basicConfig(format=FORMAT, level=logging.DEBUG)
logger = logging.getLogger(__name__)


class TestDHCPCAP:
    def test_intialize(self, dhcpcap):
        assert dhcpcap.client_mac == "00:01:02:03:04:05"
        assert dhcpcap.iface == "eth0"
        # assert dhcpcap.xid == 900000000
        # assert dhcpcap.prl == \
        #     b"\x01\x03\x06\x0f\x1f\x21\x2b\x2c\x2e\x2f\x79\xf9\xfc"
        # assert dhcpcap.client_ip == "0.0.0.0"
        # assert dhcpcap.client_port == 68
        # assert dhcpcap.server_mac == "ff:ff:ff:ff:ff:ff"
        # assert dhcpcap.server_ip == "255.255.255.255"
        # assert dhcpcap.server_port == 67
        # assert dhcpcap.lease is None
        # assert dhcpcap.event is None

    def test_gen_discover(self, dhcpcap):
        discover = dhcpcap.gen_discover()
        logger.debug(discover)
        logger.debug(dhcp_discover)
        assert discover == dhcp_discover

    def test_handle_offer(self, dhcpcap):
        dhcpcap.handle_offer(dhcp_offer)
        lease = dhcpcap.lease
        assert lease == LEASE_REQUEST

    def test_gen_request(self, dhcpcap):
        dhcpcap.lease = LEASE_REQUEST
        request = dhcpcap.gen_request()
        assert request == dhcp_request

    def test_handle_ack(self, dhcpcap):
        dhcpcap.lease = LEASE_REQUEST
        dhcpcap.handle_ack(dhcp_ack, datetime(2017, 6, 23))
        lease = dhcpcap.lease
        assert lease == LEASE_ACK



# -*- coding: utf-8 -*-
# vim:ts=4:sw=4:expandtab
# Copyright 2016, 2017 juga (juga at riseup dot net), MIT license.
"""Logging configuration."""
import logging
import sys

LOGGING = {
    'version': 1,
    'disable_existing_loggers': True,
    'formatters': {
        'verbose': {
            'format': '%(asctime)s %(levelname)s'
                      ' %(filename)s:%(lineno)s -'
                      ' %(funcName)s - %(message)s',
            'datefmt': '%Y-%m-%d %H:%M:%S'
        },
        'simple': {
            'format': "%(message)s",
        },
        'sys': {
            'format': "%(module)s[%(process)s]: "
                      "%(message)s"
        }
    },
    'handlers': {
        'stdout': {
            'class': 'logging.StreamHandler',
            'stream': sys.stdout,
            'formatter': 'verbose',
            'level': 'DEBUG',
        },
        'stdoutscapy': {
            'class': 'logging.StreamHandler',
            'stream': sys.stdout,
            'formatter': 'simple',
            'level': 'DEBUG',
        },
        'syslog': {
            'class': 'logging.handlers.SysLogHandler',
            'address': '/dev/log',
            'formatter': 'sys',
            'level': 'INFO',
        },
    },
    'loggers': {
        'dhcpcanon': {
            'handlers': ['syslog', 'stdout'],
            'level': logging.INFO,
            'propagate': False
        },
        "scapy": {
            'handlers': ['stdoutscapy'],
            'level': logging.DEBUG,
            'propagate': False
        },
        # next ones set to ERROR to disable
        # WARNING: Failed to execute tcpdump.
        "scapy.interactive": {
            'handlers': ['stdoutscapy'],
            'level': logging.ERROR,
            'propagate': False
        },
        "scapy.runtime": {
            'handlers': ['stdoutscapy'],
            'level': logging.ERROR,
            'propagate': False
        },
        "scapy.loading": {
            'handlers': ['stdoutscapy'],
            'level': logging.ERROR,
            'propagate': False
        }
    }
}



# -*- coding: utf-8 -*-
# vim:ts=4:sw=4:expandtab 2
# Copyright 2016, 2017 juga (juga at riseup dot net), MIT license.
"""Netowrk utils for the DHCP client implementation of the Anonymity Profile
([:rfc:`7844`])."""
import logging
import os.path
import subprocess

from dbus import SystemBus, Interface, DBusException
from pyroute2 import IPRoute
from pyroute2.netlink import NetlinkError

from .constants import RESOLVCONF, RESOLVCONF_ADMIN

logger = logging.getLogger(__name__)


def set_net(lease):
    ipr = IPRoute()
    try:
        index = ipr.link_lookup(ifname=lease.interface)[0]
    except IndexError as e:
        logger.error('Interface %s not found, can not set IP.',
                     lease.interface)
    try:
        ipr.addr('add', index, address=lease.address,
                 mask=int(lease.subnet_mask_cidr))
    except NetlinkError as e:
        if ipr.get_addr(index=index)[0].\
                get_attrs('IFA_ADDRESS')[0] == lease.address:
            logger.debug('Interface %s is already set to IP %s' %
                         (lease.interface, lease.address))
        else:
            logger.error(e)
    else:
        logger.debug('Interface %s set to IP %s' %
                     (lease.interface, lease.address))
    try:
        ipr.route('add', dst='0.0.0.0', gateway=lease.router, oif=index)
    except NetlinkError as e:
        if ipr.get_routes(table=254)[0].\
                get_attrs('RTA_GATEWAY')[0] == lease.router:
            logger.debug('Default gateway is already set to %s' %
                         (lease.router))
        else:
            logger.error(e)
    else:
        logger.debug('Default gateway set to %s', lease.router)
    ipr.close()
    set_dns(lease)


def set_dns(lease):
    if systemd_resolved_status() is True:
        set_dns_systemd_resolved(lease)
    elif os.path.exists(RESOLVCONF_ADMIN):
        set_dns_resolvconf_admin(lease)
    elif os.path.exists(RESOLVCONF):
        set_dns_resolvconf(lease)


def set_dns_resolvconf_admin(lease):
    cmd = [RESOLVCONF_ADMIN, 'add', lease.interface, lease.name_server]
    proc = subprocess.Popen(cmd, stdout=subprocess.PIPE,
                            stderr=subprocess.PIPE)
    try:
        (stdout, stderr) = proc.communicate()
        return True
    except TypeError as e:
        logger.error(e)
    return False


def set_dns_resolvconf(lease):
    cmd = [RESOLVCONF, '-a', lease.interface]
    proc = subprocess.Popen(cmd, stdin=subprocess.PIPE,
                            stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    stdin = '\n'.join(['nameserver ' + nm for nm in
                       lease.name_server.split()])
    stdin = str.encode(stdin)
    try:
        (stdout, stderr) = proc.communicate(stdin)
        return True
    except TypeError as e:
        logger.error(e)
    return False


def set_dns_systemd_resolved(lease):
    # NOTE: if systemd-resolved is not already running, we might not want to
    # run it in case there's specific system configuration for other resolvers
    ipr = IPRoute()
    index = ipr.link_lookup(ifname=lease.interface)[0]
    # Construct the argument to pass to DBUS.
    # the equivalent argument for:
    # busctl call org.freedesktop.resolve1 /org/freedesktop/resolve1 \
    # org.freedesktop.resolve1.Manager SetLinkDNS 'ia(iay)' 2 1 2 4 1 2 3 4
    # is SetLinkDNS(2, [(2, [8, 8, 8, 8])]_
    iay = [(2, [int(b) for b in ns.split('.')])
           for ns in lease.name_server.split()]
    #        if '.' in ns
    #        else (10, [ord(x) for x in
    #            socket.inet_pton(socket.AF_INET6, ns)])
    bus = SystemBus()
    resolved = bus.get_object('org.freedesktop.resolve1',
                              '/org/freedesktop/resolve1')
    manager = Interface(resolved,
                        dbus_interface='org.freedesktop.resolve1.Manager')
    try:
        manager.SetLinkDNS(index, iay)
        return True
    except DBusException as e:
        logger.error(e)
        return False


def systemd_resolved_status():
    bus = SystemBus()
    systemd = bus.get_object('org.freedesktop.systemd1',
                             '/org/freedesktop/systemd1')
    manager = Interface(systemd,
                        dbus_interface='org.freedesktop.systemd1.Manager')
    unit = manager.LoadUnit('sytemd-resolved.service')
    proxy = bus.get_object('org.freedesktop.systemd1', str(unit))
    r = proxy.Get('org.freedesktop.systemd1.Unit',
                  'ActiveState',
                  dbus_interface='org.freedesktop.DBus.Properties')
    if str(r) == 'active':
        return True
    return False



# -*- coding: utf-8 -*-
# vim:ts=4:sw=4:expandtab
# Copyright 2016, 2017 juga (juga at riseup dot net), MIT license.
"""."""
from dhcpcanon.dhcpcaplease import DHCPCAPLease

LEASE_INIT = DHCPCAPLease(interface='enp0s25', address='', server_id='',
                          next_server='', router='', subnet_mask='',
                          broadcast_address='', domain='', name_server='',
                          lease_time='', renewal_time='', rebinding_time='',
                          subnet_mask_cidr='', subnet='', expiry='', renew='',
                          rebind='')


LEASE_REQUEST = DHCPCAPLease(interface='eth0', address='192.168.1.23',
                             server_id='192.168.1.1',
                             next_server='192.168.1.1',
                             router='192.168.1.1', subnet_mask='255.255.255.0',
                             broadcast_address='192.168.1.255',
                             domain='localdomain',
                             name_server='192.168.1.1 8.8.8.8',
                             lease_time='43200',
                             renewal_time='21600', rebinding_time='37800',
                             subnet_mask_cidr='24', subnet='192.168.1.0',
                             expiry='', renew='', rebind='')


LEASE_ACK = DHCPCAPLease(interface='eth0', address='192.168.1.23',
                         server_id='192.168.1.1', next_server='192.168.1.1',
                         router='192.168.1.1', subnet_mask='255.255.255.0',
                         broadcast_address='192.168.1.255',
                         domain='localdomain',
                         name_server='192.168.1.1 8.8.8.8', lease_time='43200',
                         renewal_time='21600', rebinding_time='37800',
                         subnet_mask_cidr='24', subnet='192.168.1.0',
                         expiry='17-06-23 12:00:00', renew='17-06-23 06:00:00',
                         rebind='17-06-23 10:30:00')



# -*- coding: utf-8 -*-
# vim:ts=4:sw=4:expandtab 2
# Copyright 2016, 2017 juga (juga at riseup dot net), MIT license.
"""Tests for the FSM of the DHCP client implementation of the Anonymity Profile
([:rfc:`7844`]).

.. todo::

   Test for more cases:
   - delays getting OFFERs
   - delays getting ACK
   - RENEWING state
   - REBINDING state
   - lease expires

"""
import logging
import logging.config
from datetime import datetime

from scapy.automaton import Automaton
from scapy.config import conf

from dhcpcanon.conflog import LOGGING
from dhcpcanon.constants import STATES2NAMES
from dhcpcanon.dhcpcapfsm import DHCPCAPFSM
from dhcpcap_pkts import dhcp_ack, dhcp_offer
from dhcpcapfsm_objs import (fsm_bound, fsm_init, fsm_preinit, fsm_requesting,
                             fsm_selecting)

logging.config.dictConfig(LOGGING)
logger = logging.getLogger('dhcpcanon')
logger_scapy_interactive = logging.getLogger('scapy.interactive')
logger.setLevel(logging.DEBUG)
logger_scapy_interactive.setLevel(logging.DEBUG)


class DummySocket(object):
    def __init__(self, *args, **kargs):
        pass

    def send(self, pkt):
        pass

    def fileno(self):
        return 0

    def recv(self, n=0):
        return dhcp_offer

    def close(self):
        pass


class DummySocketAck(DummySocket):
    def recv(self, n=0):
        return dhcp_ack


class TestDHCPCAPFSM:
    """."""

    def test_preinit_bound(self):
        """Test FSM from PREINIT to BOUND. No delays. No script"""
        logger.debug('Test PREINIT')
        # fsm_preinit['script'].script_init(fsm_preinit['client'].lease,
        #                                   fsm_preinit['current_state'])
        # recvsock=DummySocket) will fail with python3
        conf.L2listen = DummySocket
        # for sendp:
        conf.L2socket = DummySocket
        dhcpcanon = DHCPCAPFSM(client_mac='00:01:02:03:04:05', iface='eth0',
                               xid=900000000,
                               # scriptfile='/sbin/dhcpcanon-script',
                               delay_selecting=1, timeout_select=1,
                               ll=DummySocket)
        assert dhcpcanon.dict_self() == fsm_preinit
        logger.debug('Test INIT')
        logger.debug('============')
        logger.debug('state %s', STATES2NAMES[dhcpcanon.current_state])
        # fsm_init['script'].script_init(fsm_init['client'].lease,
        #                                fsm_init['current_state'] - 1)
        logger.debug('Test start running, INIT')
        try:
            dhcpcanon.next()
        except Automaton.Singlestep as err:
            logger.debug('Singlestep %s in state %s', err,
                         dhcpcanon.current_state)
        assert dhcpcanon.dict_self() == fsm_init
        logger.debug('Test SELECTING')
        logger.debug('===============')
        logger.debug('State %s', STATES2NAMES[dhcpcanon.current_state])
        logger.debug('Num offers %s', len(dhcpcanon.offers))
        # fsm_selecting['script'].script_init(fsm_init['client'].lease,
        #                                     'PREINIT')
        logger.debug('Test timeout selecting %s',
                     dhcpcanon.get_timeout(dhcpcanon.current_state,
                                           dhcpcanon.timeout_selecting))
        # FIXME:110 why is needed here to press enter to don't retransmit?
        try:
            dhcpcanon.next()
        except Automaton.Singlestep as err:
            logger.debug('Singlestep %s in state %s', err,
                         dhcpcanon.current_state)
        # os.kill(os.getpid(), signal.SIGCONT)
        # logger.debug(dhcpcanon.dict_self()['script'])
        # logger.debug(fsm_selecting['script'])
        # TODO: case when offer is not received,
        # and next step is selecting again
        if len(dhcpcanon.offers) < 1:
            logger.debug('Offer not received, tests are not complete yet.')
            return
        # assert dhcpcanon.dict_self()['script'] == fsm_selecting['script']
        assert dhcpcanon.dict_self()['client'].lease == \
            fsm_selecting['client'].lease
        assert dhcpcanon.dict_self()['client'] == \
            fsm_selecting['client']
        # with mock.patch('dhcpcanon.timers.nowutc',
        #                 return_value=datetime(2017, 6, 23)):
        dhcpcanon.time_sent_request = datetime(2017, 6, 23)
        assert dhcpcanon.dict_self() == fsm_selecting
        logger.debug('Test REQUESTING')
        logger.debug('=================')
        logger.debug('State %s', STATES2NAMES[dhcpcanon.current_state])
        # dummy socket that will receive an ACK
        dhcpcanon.listen_sock = DummySocketAck()
        # fsm_requesting['script'].script_init(fsm_init['client'].lease,
        #                                      'PREINIT')
        try:
            dhcpcanon.next()
        except Automaton.Singlestep as err:
            logger.debug('Singlestep %s in state %s', err,
                         dhcpcanon.current_state)

        # assert dhcpcanon.dict_self()['script'] == fsm_requesting['script']
        # set the timers accourding to the time pkt sent
        # dhcpcanon.client.lease.set_times(datetime(2017, 6, 23))
        dhcpcanon.set_timers()
        assert dhcpcanon.dict_self()['client'].lease == \
            fsm_requesting['client'].lease
        assert dhcpcanon.dict_self()['client'] == \
            fsm_requesting['client']
        assert dhcpcanon.dict_self() == fsm_requesting
        logger.debug('Test BOUND')
        logger.debug('============')
        logger.debug('State %s', STATES2NAMES[dhcpcanon.current_state])
        # fsm_bound['script'].script_init(fsm_bound['client'].lease,
        #                                 'BOUND')
        try:
            dhcpcanon.next()
        except Automaton.Singlestep as err:
            logger.debug('Singlestep %s in state %s', err,
                         dhcpcanon.current_state)
        # assert dhcpcanon.dict_self()['script'] == fsm_bound['script']
        assert dhcpcanon.dict_self()['client'].lease == \
            fsm_bound['client'].lease
        assert dhcpcanon.dict_self()['client'] == \
            fsm_bound['client']
        assert dhcpcanon.dict_self() == fsm_bound



# -*- coding: utf-8 -*-
# vim:ts=4:sw=4:expandtab 2
# Copyright 2016, 2017 juga (juga at riseup dot net), MIT license.
"""."""
from datetime import datetime

from dhcpcanon.clientscript import ClientScript
from dhcpcanon.dhcpcap import DHCPCAP
from dhcpcanon.dhcpcaplease import DHCPCAPLease

fsm_preinit = {
    'current_state': 0,
    'request_attempts': 0,
    'discover_attempts': 0,
    'time_sent_request': None,
    'client': DHCPCAP(iface='eth0', client_mac='00:01:02:03:04:05',
                      client_ip='0.0.0.0', client_port=68, xid=900000000,
                      server_mac='ff:ff:ff:ff:ff:ff',
                      server_ip='255.255.255.255', server_port=67,
                      event=None,
                      lease=DHCPCAPLease(address='', server_id='',
                                         next_server='', router='',
                                         subnet_mask='',
                                         broadcast_address='',
                                         domain='', name_server='',
                                         subnet='', lease_time='',
                                         renewal_time='',
                                         rebinding_time='',
                                         interface='eth0',
                                         subnet_mask_cidr='',
                                         network='', expiry='',
                                         renew='', rebind=''),),
    'script': ClientScript(scriptname='/sbin/dhcpcanon-script',
                           env={'rebind': '', 'medium': '',
                                'broadcast_address': '', 'renew': '',
                                'domain_name_servers': '',
                                'dhcp_server_identifier': '',
                                'pid': '25318', 'domain_name': '',
                                'network_number': '', 'subnet_mask': '',
                                'reason': 'INIT',
                                'client': 'dhcpcanon',
                                'next_server': '', 'expiry': '',
                                'routers': '', 'interface': 'eth0',
                                'dhcp_lease_time': '', 'ip_address': '',
                                'dhcp_renewal_time': '',
                                'dhcp_rebinding_time': ''}),
    }


# client after send discover in state 1
fsm_init = {
    'current_state': 1,
    'request_attempts': 0,
    'discover_attempts': 1,
    'time_sent_request': None,
    'client': DHCPCAP(iface='eth0', client_mac='00:01:02:03:04:05',
                      client_ip='0.0.0.0', client_port=68, xid=900000000,
                      server_mac='ff:ff:ff:ff:ff:ff',
                      server_ip='255.255.255.255', server_port=67,
                      event=None,
                      lease=DHCPCAPLease(address='', server_id='',
                                         next_server='', router='',
                                         subnet_mask='',
                                         broadcast_address='',
                                         domain='', name_server='',
                                         subnet='', lease_time='',
                                         renewal_time='',
                                         rebinding_time='',
                                         interface='eth0',
                                         subnet_mask_cidr='',
                                         network='', expiry='',
                                         renew='', rebind=''),),
    'script': ClientScript(scriptname='/sbin/dhcpcanon-script',
                           env={'rebind': '', 'medium': '',
                                'broadcast_address': '', 'renew': '',
                                'domain_name_servers': '',
                                'dhcp_server_identifier': '',
                                'pid': '25318', 'domain_name': '',
                                'network_number': '', 'subnet_mask': '',
                                'reason': 'PREINIT',
                                'client': '',
                                'next_server': '', 'expiry': '',
                                'routers': '', 'interface': '',
                                'dhcp_lease_time': '', 'ip_address': '',
                                'dhcp_renewal_time': '',
                                'dhcp_rebinding_time': ''}),
    }


fsm_selecting = {
    'current_state': 2,
    'request_attempts': 0,
    'discover_attempts': 1,
    'time_sent_request': datetime(2017, 6, 23),
    'client': DHCPCAP(iface='eth0',
                      client_mac='00:01:02:03:04:05',
                      client_ip='0.0.0.0', client_port=68, xid=900000000,
                      server_mac='ff:ff:ff:ff:ff:ff',
                      server_ip='255.255.255.255', server_port=67,
                      event=None,
                      lease=DHCPCAPLease(address='192.168.1.23',
                                         server_id='192.168.1.1',
                                         next_server='192.168.1.1',
                                         router='192.168.1.1',
                                         subnet_mask='255.255.255.0',
                                         broadcast_address='192.168.1.255',
                                         domain='localdomain',
                                         name_server='192.168.1.1',
                                         subnet='192.168.1.0',
                                         lease_time='43200',
                                         renewal_time='21600',
                                         rebinding_time='37800',
                                         interface='eth0',
                                         subnet_mask_cidr='24',
                                         network='', expiry='',
                                         renew='', rebind=''),),
    'script': ClientScript(scriptname='/sbin/dhcpcanon-script',
                           env={'rebind': '', 'medium': '',
                                'broadcast_address': '',
                                'renew': '',
                                'domain_name_servers': '',
                                'dhcp_server_identifier': '',
                                'pid': '25318', 'domain_name': '',
                                'network_number': '',
                                'subnet_mask': '',
                                'reason': 'PREINIT',
                                'client': 'dhcpcanon',
                                'next_server': '', 'expiry': '',
                                'routers': '',
                                'interface': 'eth0',
                                'dhcp_lease_time': '',
                                'ip_address': '192.168.1.23',
                                'dhcp_renewal_time': '21600',
                                'dhcp_rebinding_time': '37800'}),
    }


fsm_requesting = {
    'current_state': 3,
    'request_attempts': 0,
    'discover_attempts': 1,
    'time_sent_request': datetime(2017, 6, 23),
    'client': DHCPCAP(iface='eth0', client_mac='00:01:02:03:04:05',
                      client_ip='192.168.1.23', client_port=68, xid=900000000,
                      server_mac='00:0a:0b:0c:0d:0f',
                      server_ip='192.168.1.1', server_port=67,
                      event=4,
                      lease=DHCPCAPLease(address='192.168.1.23',
                                         server_id='192.168.1.1',
                                         next_server='192.168.1.1',
                                         router='192.168.1.1',
                                         subnet_mask='255.255.255.0',
                                         broadcast_address='192.168.1.255',
                                         domain='localdomain',
                                         name_server='192.168.1.1',
                                         subnet='192.168.1.0',
                                         lease_time='43200',
                                         renewal_time='21600',
                                         rebinding_time='37800',
                                         interface='eth0',
                                         subnet_mask_cidr='24',
                                         network='',
                                         expiry='17-06-23 12:00:00',
                                         renew='17-06-23 06:00:00',
                                         rebind='17-06-23 10:30:00'),),
    'script': ClientScript(scriptname='/sbin/dhcpcanon-script',
                           env={'rebind': '', 'medium': '',
                                'broadcast_address': '',
                                'renew': '',
                                'domain_name_servers': '',
                                'dhcp_server_identifier': '',
                                'pid': '25318', 'domain_name': '',
                                'network_number': '',
                                'subnet_mask': '',
                                'reason': 'PREINIT',
                                'client': 'dhcpcanon',
                                'next_server': '',
                                'expiry': '', 'routers': '',
                                'interface': 'eth0',
                                'dhcp_lease_time': '',
                                'ip_address': '',
                                'dhcp_renewal_time': '',
                                'dhcp_rebinding_time': ''}),
    }


fsm_bound = {
    'current_state': 4,
    'request_attempts': 0,
    'discover_attempts': 1,
    'time_sent_request': datetime(2017, 6, 23),
    'client': DHCPCAP(iface='eth0', client_mac='00:01:02:03:04:05',
                      client_ip='192.168.1.23', client_port=68, xid=900000000,
                      server_mac='00:0a:0b:0c:0d:0f',
                      server_ip='192.168.1.1', server_port=67,
                      event=4,
                      lease=DHCPCAPLease(address='192.168.1.23',
                                         server_id='192.168.1.1',
                                         next_server='192.168.1.1',
                                         router='192.168.1.1',
                                         subnet_mask='255.255.255.0',
                                         broadcast_address='192.168.1.255',
                                         domain='localdomain',
                                         name_server='192.168.1.1',
                                         subnet='192.168.1.0',
                                         lease_time='43200',
                                         renewal_time='21600',
                                         rebinding_time='37800',
                                         interface='eth0',
                                         subnet_mask_cidr='24',
                                         network='',
                                         expiry='17-06-23 12:00:00',
                                         renew='17-06-23 06:00:00',
                                         rebind='17-06-23 10:30:00'),),
    'script': ClientScript(scriptname='/sbin/dhcpcanon-script',
                           env={'rebind': '17-06-23 10:30:00', 'medium': '',
                                'broadcast_address': '192.168.1.255',
                                'renew': '17-06-23 06:00:00',
                                'domain_name_servers': '192.168.1.1',
                                'dhcp_server_identifier': '192.168.1.1',
                                'pid': '25318', 'domain_name': 'localdomain',
                                'network_number': '24',
                                'subnet_mask': '255.255.255.0',
                                'reason': 'PREINIT',
                                'client': 'dhcpcanon',
                                'next_server': '192.168.1.1',
                                'expiry': '17-06-23 12:00:00',
                                'routers': '192.168.1.1',
                                'interface': 'eth0',
                                'dhcp_lease_time': '43200',
                                'ip_address': '',
                                'dhcp_renewal_time': '21600',
                                'dhcp_rebinding_time': '37800'}),
    }
# without script
###############################################################################

# -*- coding: utf-8 -*-
# vim:ts=4:sw=4:expandtab 2
# Copyright 2016, 2017 juga (juga at riseup dot net), MIT license.
"""."""
from datetime import datetime

from dhcpcanon.clientscript import ClientScript
from dhcpcanon.dhcpcap import DHCPCAP
from dhcpcanon.dhcpcaplease import DHCPCAPLease

fsm_preinit = {
    'current_state': 0,
    'request_attempts': 0,
    'discover_attempts': 0,
    'time_sent_request': None,
    'client': DHCPCAP(iface='eth0', client_mac='00:01:02:03:04:05',
                      client_ip='0.0.0.0', client_port=68, xid=900000000,
                      server_mac='ff:ff:ff:ff:ff:ff',
                      server_ip='255.255.255.255', server_port=67,
                      event=None,
                      lease=DHCPCAPLease(address='', server_id='',
                                         next_server='', router='',
                                         subnet_mask='',
                                         broadcast_address='',
                                         domain='', name_server='',
                                         subnet='', lease_time='',
                                         renewal_time='',
                                         rebinding_time='',
                                         interface='eth0',
                                         subnet_mask_cidr='',
                                         network='', expiry='',
                                         renew='', rebind=''),),
    'script': None,
    }


# client after send discover in state 1
fsm_init = {
    'current_state': 1,
    'request_attempts': 0,
    'discover_attempts': 1,
    'time_sent_request': None,
    'client': DHCPCAP(iface='eth0', client_mac='00:01:02:03:04:05',
                      client_ip='0.0.0.0', client_port=68, xid=900000000,
                      server_mac='ff:ff:ff:ff:ff:ff',
                      server_ip='255.255.255.255', server_port=67,
                      event=None,
                      lease=DHCPCAPLease(address='', server_id='',
                                         next_server='', router='',
                                         subnet_mask='',
                                         broadcast_address='',
                                         domain='', name_server='',
                                         subnet='', lease_time='',
                                         renewal_time='',
                                         rebinding_time='',
                                         interface='eth0',
                                         subnet_mask_cidr='',
                                         network='', expiry='',
                                         renew='', rebind=''),),
    'script': None,
    }


fsm_selecting = {
    'current_state': 2,
    'request_attempts': 0,
    'discover_attempts': 1,
    'time_sent_request': datetime(2017, 6, 23),
    'client': DHCPCAP(iface='eth0',
                      client_mac='00:01:02:03:04:05',
                      client_ip='0.0.0.0', client_port=68, xid=900000000,
                      server_mac='ff:ff:ff:ff:ff:ff',
                      server_ip='255.255.255.255', server_port=67,
                      event=None,
                      lease=DHCPCAPLease(address='192.168.1.23',
                                         server_id='192.168.1.1',
                                         next_server='192.168.1.1',
                                         router='192.168.1.1',
                                         subnet_mask='255.255.255.0',
                                         broadcast_address='192.168.1.255',
                                         domain='localdomain',
                                         name_server='192.168.1.1 8.8.8.8',
                                         subnet='192.168.1.0',
                                         lease_time='43200',
                                         renewal_time='21600',
                                         rebinding_time='37800',
                                         interface='eth0',
                                         subnet_mask_cidr='24',
                                         network='', expiry='',
                                         renew='', rebind=''),),
    'script': None,
    }


fsm_requesting = {
    'current_state': 3,
    'request_attempts': 0,
    'discover_attempts': 1,
    'time_sent_request': datetime(2017, 6, 23),
    'client': DHCPCAP(iface='eth0', client_mac='00:01:02:03:04:05',
                      client_ip='192.168.1.23', client_port=68, xid=900000000,
                      server_mac='00:0a:0b:0c:0d:0f',
                      server_ip='192.168.1.1', server_port=67,
                      event=4,
                      lease=DHCPCAPLease(address='192.168.1.23',
                                         server_id='192.168.1.1',
                                         next_server='192.168.1.1',
                                         router='192.168.1.1',
                                         subnet_mask='255.255.255.0',
                                         broadcast_address='192.168.1.255',
                                         domain='localdomain',
                                         name_server='192.168.1.1 8.8.8.8',
                                         subnet='192.168.1.0',
                                         lease_time='43200',
                                         renewal_time='21600',
                                         rebinding_time='37800',
                                         interface='eth0',
                                         subnet_mask_cidr='24',
                                         network='',
                                         expiry='17-06-23 12:00:00',
                                         renew='17-06-23 06:00:00',
                                         rebind='17-06-23 10:30:00'),),
    'script': None,
    }


fsm_bound = {
    'current_state': 4,
    'request_attempts': 0,
    'discover_attempts': 1,
    'time_sent_request': datetime(2017, 6, 23),
    'client': DHCPCAP(iface='eth0', client_mac='00:01:02:03:04:05',
                      client_ip='192.168.1.23', client_port=68, xid=900000000,
                      server_mac='00:0a:0b:0c:0d:0f',
                      server_ip='192.168.1.1', server_port=67,
                      event=4,
                      lease=DHCPCAPLease(address='192.168.1.23',
                                         server_id='192.168.1.1',
                                         next_server='192.168.1.1',
                                         router='192.168.1.1',
                                         subnet_mask='255.255.255.0',
                                         broadcast_address='192.168.1.255',
                                         domain='localdomain',
                                         name_server='192.168.1.1 8.8.8.8',
                                         subnet='192.168.1.0',
                                         lease_time='43200',
                                         renewal_time='21600',
                                         rebinding_time='37800',
                                         interface='eth0',
                                         subnet_mask_cidr='24',
                                         network='',
                                         expiry='17-06-23 12:00:00',
                                         renew='17-06-23 06:00:00',
                                         rebind='17-06-23 10:30:00'),),
    'script': None,
    }



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# vim:ts=4:sw=4:expandtab 2
# Copyright 2016, 2017 juga (juga at riseup dot net), MIT license.
"""DCHP client implementation of the anonymity profile (RFC7844)."""

import argparse
import logging
import logging.config

from lockfile.pidlockfile import (PIDLockFile, AlreadyLocked,
                                  LockTimeout, LockFailed)
from scapy.config import conf

# in python3 this seems to be the only way to to disable:
# WARNING: Failed to execute tcpdump.
conf.logLevel = logging.ERROR

from . import __version__
from .conflog import LOGGING
from .constants import (CLIENT_PORT, SERVER_PORT, SCRIPT_PATH, PID_PATH)
from .dhcpcapfsm import DHCPCAPFSM

logging.config.dictConfig(LOGGING)
logger = logging.getLogger('dhcpcanon')


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument('interface', nargs='?',
                        help='interface to configure with DHCP')
    parser.add_argument('-v', '--verbose',
                        help='Set logging level to debug',
                        action='store_true')
    parser.add_argument('--version', action='version',
                        help='version',
                        version='%(prog)s ' + __version__)
    parser.add_argument('-s', '--delay_selecting',
                        help='Selecting starts after a ramdon delay.',
                        action='store_true')
    # options to looks like dhclient
    parser.add_argument(
        '-sf', metavar='script-file', nargs='?',
        const=SCRIPT_PATH,
        help='Path to the network configuration script invoked by '
             'dhcpcanon when it gets a lease. Without this option '
             'dhcpcanon will configure the network by itself.'
             'If unspecified, the '
             'default /sbin/dhcpcanon-script is used, which is a copy of'
             'dhclient-script(8) for a description of this file.'
             'If dhcpcanon is running with NetworkManager, it will'
             'be called with the script nm-dhcp-helper.')
    parser.add_argument(
        '-pf', metavar='pid-file', nargs='?',
        const=PID_PATH,
        help='Path to the process ID file. If unspecified, the'
             'default /var/run/dhcpcanon.pid is used. '
             'This option is used by NetworkManager to check whether '
             'dhcpcanon is already running.')
    args = parser.parse_args()
    logger.debug('args %s', args)

    # do not put interfaces in promiscuous mode
    conf.sniff_promisc = conf.promisc = 0
    conf.checkIPaddr = 1

    if args.verbose:
        logger.setLevel(logging.DEBUG)
    logger.debug('args %s', args)
    if args.interface:
        conf.iface = args.interface
    logger.debug('interface %s' % conf.iface)
    if args.pf is not None:
        # This is only needed for nm
        pf = PIDLockFile(args.pf, timeout=5)
        try:
            pf.acquire()
            logger.debug('using pid file %s', pf)
        except AlreadyLocked as e:
            pf.break_lock()
            pf.acquire()
        except (LockTimeout, LockFailed) as e:
            logger.error(e)
    dhcpcap = DHCPCAPFSM(iface=conf.iface,
                         server_port=SERVER_PORT,
                         client_port=CLIENT_PORT,
                         scriptfile=args.sf,
                         delay_selecting=args.delay_selecting)
    dhcpcap.run()


if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim:ts=4:sw=4:expandtab
# Copyright 2016, 2017 juga (juga at riseup dot net), MIT license.

"""Setup."""
import subprocess

from setuptools import find_packages, setup

import dhcpcanon


def systemd_unit_dir():
    cmd = ['pkg-config',  '--variable', 'systemdsystemunitdir', 'systemd']
    proc = subprocess.Popen(cmd, stdout=subprocess.PIPE,
                            stderr=subprocess.PIPE)
    (stdout, stderr) = proc.communicate()
    if proc.returncode != 0 or stdout is None:
        return None  # systemd not found
    if isinstance(stdout.strip(), bytes):
        return stdout.strip().decode('utf-8')
    return stdout.strip()


def systemd_tmpfiles_dir():
    # There doesn't seem to be a specific pkg-config variable for this
    cmd = ['pkg-config',  '--variable', 'prefix', 'systemd']
    proc = subprocess.Popen(cmd, stdout=subprocess.PIPE,
                            stderr=subprocess.PIPE)
    (stdout, stderr) = proc.communicate()
    if proc.returncode != 0 or stdout is None:
        return None  # systemd not found
    d = stdout.strip()
    if isinstance(d, bytes):
        d = d.decode('utf-8')
    return d + '/lib/tmpfiles.d'


if systemd_unit_dir():
    data_files = [
        (systemd_unit_dir(), ['systemd/dhcpcanon.service']),
        (systemd_tmpfiles_dir(), ['tmpfiles.d/dhcpcanon.conf']),
        ('sbin', ["sbin/dhcpcanon-script"])
    ]
else:
    data_files = []
setup(
    name='dhcpcanon',
    version=dhcpcanon.__version__,
    description=dhcpcanon.__description__,
    long_description=dhcpcanon.__long_description__,
    author=dhcpcanon.__author__,
    author_email=dhcpcanon.__author_mail__,
    license='MIT',
    url=dhcpcanon.__website__,
    packages=find_packages(exclude=['contrib', 'docs', 'tests*']),
    install_requires=[
        "attrs>=16.3",
        "dbus-python>=1.2",
        "netaddr>=0.7",
        "lockfile>=0.12",
        "pip>=8.1",
        "pyroute2>=0.3",
        'scapy-python3>=0.20',
    ],
    python_requires=">=3.5",
    extras_require={
        'dev': ['ipython', 'pyflakes', 'pep8'],
        'test': ['coverage', 'coveralls', 'codecov', 'tox', 'pytest'],
        'doc': ['sphinx', 'pylint']
    },
    entry_points={
        'console_scripts': [
            'dhcpcanon = dhcpcanon.dhcpcanon:main',
        ]
    },
    # setup_requires=['pytest-runner'],
    tests_require=['pytest'],
    # NOTE: not installing system files as the user might want to install them
    # in a custom prefix or without systemd
    # data_files=data_files,
    zip_safe=False,
    include_package_data=True,
    keywords='python scapy dhcp RFC7844 RFC2131 anonymity',
    classifiers=[
        'Development Status :: 3 - Alpha',
        "Environment :: Console",
        'Intended Audience :: End Users/Desktop',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3',
        'Topic :: System :: Networking',
    ],
)
